@@ -258,6 +258,9 @@ getPosByTargetForPairs <- function(queryObj, targetObj,
   if(length(mcols(queryObj)$color)){
     res$color <- mcols(queryObj)$color[res$index]
   }
+  if(length(mcols(queryObj)$lwd)){
+    res$lwd <- mcols(queryObj)$lwd[res$index]
+  }
   return(res)
 }
 # map score to segments lwd
@@ -265,6 +268,7 @@ createSegmentGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     lwd.maxGenomicSigs = 8, alpha = 1, length.out,
+    resizeFactor = 1,
     ...) {
   if(lwd.maxGenomicSigs[1]<2){
     warning('Small lwd.maxGenomicSigs will lead to some errors when plot it.')
@@ -324,34 +328,48 @@ createSegmentGeometry <- function(
       )
     }
     names(color) <- genomicSigLwd
-    genomic_signal <- lapply(genomicSigLwd, function(lwd) {
-      idx <- which(GenoSig$lwd == lwd)
-      threeJsGeometry(
-        x = as.numeric(t(as.matrix(mcols(GenoSig)[idx, c("x0", "x1"),
-          drop = FALSE
-        ]))),
-        y = as.numeric(t(as.matrix(mcols(GenoSig)[idx, c("y0", "y1"),
-          drop = FALSE
-        ]))),
-        z = as.numeric(t(as.matrix(mcols(GenoSig)[idx, c("z0", "z1"),
-          drop = FALSE
-        ]))),
-        type = "segment",
-        colors = if(length(GenoSig$color)) GenoSig$color[idx] else
-          color[as.character(lwd)],
-        tag = tag,
-        rotation = rotation,
-        properties = list(
-          size = lwd,
-          alpha = alpha
-        )
-      )
-    })
-    names(genomic_signal) <- paste0(name, "_lwd_", genomicSigLwd)
-    genomic_signal
   } else {
-    NULL
+    if(length(GenoSig$lwd)==length(GenoSig)){
+      genomicSigLwd <- unique(GenoSig$lwd)
+    }else{
+      genomicSigLwd <- lwd.maxGenomicSigs
+      GenoSig$lwd <- genomicSigLwd
+    }
+    if(length(genomicSigLwd)<=length(color)){
+      if(length(genomicSigLwd)<length(color)) {
+        color <- color[-seq.int(length(color)-length(genomicSigLwd))]
+      }
+      names(color) <- genomicSigLwd
+    }else{
+      stop('The input color length is smaller than the number of distinct line width.')
+    }
   }
+  genomic_signal <- lapply(genomicSigLwd, function(lwd) {
+    idx <- which(GenoSig$lwd == lwd)
+    threeJsGeometry(
+      x = as.numeric(t(as.matrix(mcols(GenoSig)[idx, c("x0", "x1"),
+                                                drop = FALSE
+      ]))),
+      y = as.numeric(t(as.matrix(mcols(GenoSig)[idx, c("y0", "y1"),
+                                                drop = FALSE
+      ]))),
+      z = as.numeric(t(as.matrix(mcols(GenoSig)[idx, c("z0", "z1"),
+                                                drop = FALSE
+      ]))),
+      type = "segment",
+      colors = if(length(GenoSig$color)) GenoSig$color[idx] else
+        color[as.character(lwd)],
+      tag = tag,
+      rotation = rotation,
+      properties = list(
+        size = lwd,
+        alpha = alpha,
+        resizeFactor = resizeFactor
+      )
+    )
+  })
+  names(genomic_signal) <- paste0(name, "_lwd_", genomicSigLwd)
+  genomic_signal
 }
 
 getXYZmean <- function(GenoSig) {
@@ -379,6 +397,7 @@ createCircleGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     radius = 8, maxVal = 1, thetaStart = 0,
+    resizeFactor = 1,
     ...) {
   GenoSig$score <- 2 * pi * GenoSig$score / maxVal[1]
   if (revGenoSig) GenoSig$score <- 2 * pi - GenoSig$score
@@ -403,7 +422,8 @@ createCircleGeometry <- function(
           properties = list(
             radius = r,
             thetaStart = ts,
-            thetaLength = pct
+            thetaLength = pct,
+            resizeFactor = resizeFactor
           )
         )
       }, SIMPLIFY = FALSE
@@ -424,7 +444,8 @@ createCircleGeometry <- function(
           properties = list(
             radius = r,
             thetaStart = ts,
-            thetaLength = GenoSig$score[idx]
+            thetaLength = GenoSig$score[idx],
+            resizeFactor = resizeFactor
           )
         )
       }, SIMPLIFY = FALSE
@@ -466,6 +487,7 @@ createSphereGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     radius = 8, type = "sphere",
+    resizeFactor = 1,
     ...) {
   color <- mapScore2Color(GenoSig, color, genomicScoreRange)
   GenoSig <- getXYZmean(GenoSig)
@@ -481,7 +503,8 @@ createSphereGeometry <- function(
         tag = tag,
         rotation = rotation,
         properties = list(
-          radius = .radius
+          radius = .radius,
+          resizeFactor = resizeFactor
         )
       )
     }, seq_along(GenoSig), radius, color)
@@ -497,7 +520,8 @@ createSphereGeometry <- function(
       tag = tag,
       rotation = rotation,
       properties = list(
-        radius = radius[1]
+        radius = radius[1],
+        resizeFactor = resizeFactor
       )
     )
     genomic_signal <- list(genomic_signal)
@@ -512,6 +536,7 @@ createBoxGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     width, height, depth,
+    resizeFactor = 1,
     ...) {
   if (missing(width) || missing(height) || missing(depth)) {
     if (missing(width)) {
@@ -544,7 +569,8 @@ createBoxGeometry <- function(
         properties = list(
           width = wid,
           height = hgt,
-          depth = dpt
+          depth = dpt,
+          resizeFactor = resizeFactor
         )
       )
     )
@@ -562,7 +588,8 @@ createBoxGeometry <- function(
         properties = list(
           width = w,
           height = h,
-          depth = d
+          depth = d,
+          resizeFactor = resizeFactor
         )
       )
     }, seq_along(GenoSig), width, height, depth, color)
@@ -576,6 +603,7 @@ createCapsuleGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     height, radius,
+    resizeFactor = 1,
     ...) {
   if (missing(height) || missing(radius)) {
     if (missing(height)) {
@@ -603,7 +631,8 @@ createCapsuleGeometry <- function(
         rotation = rotation,
         properties = list(
           height = hgt,
-          radius = r
+          radius = r,
+          resizeFactor = resizeFactor
         )
       )
     )
@@ -620,7 +649,8 @@ createCapsuleGeometry <- function(
         rotation = rotation,
         properties = list(
           height = h,
-          radius = d
+          radius = d,
+          resizeFactor = resizeFactor
         )
       )
     }, seq_along(GenoSig), height, radius, color)
@@ -634,6 +664,7 @@ createCylinderGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     height, radiusTop, radiusBottom,
+    resizeFactor = 1,
     ...) {
   if (missing(height) || missing(radiusTop) || missing(radiusBottom)) {
     if (missing(height)) {
@@ -666,7 +697,8 @@ createCylinderGeometry <- function(
         properties = list(
           height = hgt,
           radiusTop = rt,
-          radiusBottom = rb
+          radiusBottom = rb,
+          resizeFactor = resizeFactor
         )
       )
     )
@@ -684,7 +716,8 @@ createCylinderGeometry <- function(
         properties = list(
           height = h,
           radiusTop = rt,
-          radiusBottom = rb
+          radiusBottom = rb,
+          resizeFactor = resizeFactor
         )
       )
     }, seq_along(GenoSig), height, radiusTop, radiusBottom, color)
@@ -698,6 +731,7 @@ createConeGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     height, radius,
+    resizeFactor = 1,
     ...) {
   if (missing(height) || missing(radius)) {
     if (missing(height)) {
@@ -725,7 +759,8 @@ createConeGeometry <- function(
         rotation = rotation,
         properties = list(
           height = hgt,
-          radius = r
+          radius = r,
+          resizeFactor = resizeFactor
         )
       )
     )
@@ -742,7 +777,8 @@ createConeGeometry <- function(
         rotation = rotation,
         properties = list(
           height = h,
-          radius = d
+          radius = d,
+          resizeFactor = resizeFactor
         )
       )
     }, seq_along(GenoSig), height, radius, color)
@@ -769,6 +805,7 @@ createTorusGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
     tube, radius,
+    resizeFactor = 1,
     ...) {
   if (missing(tube) || missing(radius)) {
     if (missing(tube)) {
@@ -796,7 +833,8 @@ createTorusGeometry <- function(
         rotation = rotation,
         properties = list(
           tube = hgt,
-          radius = r
+          radius = r,
+          resizeFactor = resizeFactor
         )
       )
     )
@@ -813,7 +851,8 @@ createTorusGeometry <- function(
         rotation = rotation,
         properties = list(
           tube = h,
-          radius = d
+          radius = d,
+          resizeFactor = resizeFactor
         )
       )
     }, seq_along(GenoSig), tube, radius, color)
@@ -827,7 +866,7 @@ createTorusGeometry <- function(
 createJsonGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
-    path,
+    path, resizeFactor = 1,
     ...) {
   json <- fromJSON(file = path)
   stopifnot(
@@ -846,7 +885,8 @@ createJsonGeometry <- function(
       tag = tag,
       rotation = rotation,
       properties = list(
-        json = json
+        json = json,
+        resizeFactor = resizeFactor
       )
     )
   )
@@ -922,6 +962,7 @@ mapScore2Alpha <- function(score, default=0.1, genomicScoreRange){
 createPolygonGeometry <- function(
     GenoSig, genomicScoreRange, revGenoSig,
     name, color, tag, rotation,
+    resizeFactor = 1,
     ...) {
   genomic_signal <- data.frame(
     GenoSig$x0, GenoSig$x1, GenoSig$x1_2, GenoSig$x0_2,
@@ -975,7 +1016,8 @@ createPolygonGeometry <- function(
     rotation = rotation,
     properties = list(
       alpha=rep(genomic_signal$alpha, ncol(x))[idJ],
-      indices=as.integer(indices)-1
+      indices=as.integer(indices)-1,
+      resizeFactor = resizeFactor
     )
   ))
   names(geo) <- name
