@@ -31,12 +31,13 @@
 #' htmlwidget.
 #' @importFrom stats quantile
 #' @importFrom trackViewer parseWIG
-#' @importFrom GenomeInfoDb seqnames
+#' @importFrom Seqinfo seqnames
 #' @importFrom BiocGenerics start<- end<- strand<-
 #' @importFrom utils tail
 #' @importFrom grDevices colorRampPalette
 #' @importFrom grid convertUnit grid.newpage viewport pushViewport popViewport
 #' convertWidth convertHeight xsplineGrob grobCoords convertX convertY is.unit
+#' @importFrom GenomicRanges GRangesList
 #' @export
 #' @examples
 #' obj <- readRDS(system.file("extdata", "4DNFI1UEG1HD.chr21.FLAMINGO.res.rds",
@@ -81,12 +82,14 @@ view3dStructure <- function(obj, feature.gr,
     obj <- subsetByOverlaps(obj, region)
   }
   checkSignalTransformFun(signalTransformFun)
-  stopifnot(
-    "Only work for data in a single chromsome." =
-      all(as.character(seqnames(obj)) == as.character(seqnames(obj)[1]))
-  )
   stopifnot(is.numeric(k))
   stopifnot(k == 2 || k == 3)
+  singleChr <- all(as.character(seqnames(obj)) == as.character(seqnames(obj)[1]))
+  if(k==2){
+    if(!singleChr){
+      stop("Only work for data in a single chromsome.")
+    }
+  }
   stopifnot(all(c("x", "y") %in% colnames(mcols(obj))))
   if (k == 3) {
     stopifnot("z" %in% colnames(mcols(obj)))
@@ -99,10 +102,10 @@ view3dStructure <- function(obj, feature.gr,
     feature.gr = feature.gr,
     seqn = seqn
   )
-  xlim <- range(obj$x)
-  ylim <- range(obj$y)
+  xlim <- range(obj$x, na.rm = TRUE)
+  ylim <- range(obj$y, na.rm = TRUE)
   if (length(obj$z) > 0) {
-    zlim <- range(obj$z)
+    zlim <- range(obj$z, na.rm = TRUE)
   } else {
     zlim <- c(-1, 1) ## will not use
   }
@@ -227,48 +230,110 @@ view3dStructure <- function(obj, feature.gr,
   } else {
     ## plot 3D
     # recenter the x,y,z
-    xcenter <- mean(xlim)
-    ycenter <- mean(ylim)
-    zcenter <- mean(zlim)
+    xcenter <- mean(xlim, na.rm=TRUE)
+    ycenter <- mean(ylim, na.rm=TRUE)
+    zcenter <- mean(zlim, na.rm=TRUE)
     obj$x <- obj$x - xcenter
     obj$y <- obj$y - ycenter
     obj$z <- obj$z - zcenter
-    mf <- 5 / max(abs(obj$x), abs(obj$y), abs(obj$z)) ## change to around -5:5
-    obj$x <- obj$x * mf
-    obj$y <- obj$y * mf
-    obj$z <- obj$z * mf
+    resizeFactor <- 5 / max(abs(obj$x), abs(obj$y), abs(obj$z), na.rm = TRUE) ## change to around -5:5
+    obj$x <- obj$x * resizeFactor
+    obj$y <- obj$y * resizeFactor
+    obj$z <- obj$z * resizeFactor
     
     dots <- list(...)
     ## add point cluster annotation
     if(cluster3Dpoints){
       eps <- ifelse("eps" %in% names(dots), dots$eps, 'auto')
-      clusters <- pointCluster(obj, eps = eps)
+      quite <- ifelse('quite' %in% names(dots), dots$quite, TRUE)
+      if(eps!='auto') eps <- eps * resizeFactor
+      obj.cp <- obj
+      mcols(obj.cp) <- mcols(obj)[, c('x', 'y', 'z')]
+      obj.cp <- fill_NA(obj.cp)
+      clusters <- pointCluster(obj.cp, eps = eps, resizeFactor=resizeFactor, quite = quite)
+      rm(obj.cp)
       pc <- clusterAnno(obj, clusters)
-      pc_geometries <- createPointClusterGeometries(pc, obj)
     }
     ## spline smooth for each bin with 30 points
     if('resolution' %in% names(dots)){
-      obj <- smooth3dPoints(obj, dots$resolution)
+      resolution <- dots$resolution
     }else{
-      obj <- smooth3dPoints(obj, 30)
+      resolution <- 30
+    }
+    tryCatch({
+      if(singleChr){
+        obj <- smooth3dPoints(obj, resolution)
+      }else{
+        obj <- lapply(split(obj, as.character(seqnames(obj))),
+                      smooth3dPoints, resolution=resolution)
+      }
+    }, error=function(e){
+      stop('Can not smooth the 3D model. Please try to add parameter resolution=1')
+    })
+    
+    geometries <- list() ## list to save all geometries to plot
+    if(singleChr){
+      geometries$backbone <- threeJsGeometry(
+        x = c(obj$x0, obj$x1[length(obj)]),
+        y = c(obj$y0, obj$y1[length(obj)]),
+        z = c(obj$z0, obj$z1[length(obj)]),
+        colors = col.backbone,
+        type = "line",
+        tag = "backbone",
+        properties = list(
+          size = lwd.backbone,
+          target = unname(as.character(ranges(obj))),
+          seqn = seqn,
+          resizeFactor = resizeFactor
+        )
+      )
+    }else{
+      if(length(col.backbone)>length(obj)){
+        this.col.backbone <- col.backbone[seq_along(obj)]
+      }else{
+        this.col.backbone <- col.backbone
+      }
+      if(length(lwd.backbone)>length(obj)){
+        this.lwd.backbone <- lwd.backbone[seq_along(obj)]
+      }else{
+        this.lwd.backbone <- lwd.backbone
+      }
+      if(all(names(obj) %in% names(this.col.backbone))){
+        this.col.backbone <- this.col.backbone[names(obj)]
+      }
+      if(all(names(obj) %in% names(this.lwd.backbone))){
+        this.lwd.backbone <- this.lwd.backbone[names(obj)]
+      }
+      backbone <- mapply(function(.obj, .col, .lwd, .seqn){
+        threeJsGeometry(
+          x = c(.obj$x0, .obj$x1[length(.obj)]),
+          y = c(.obj$y0, .obj$y1[length(.obj)]),
+          z = c(.obj$z0, .obj$z1[length(.obj)]),
+          colors = .col,
+          type = "line",
+          tag = "backbone",
+          properties = list(
+            size = .lwd,
+            target = unname(as.character(ranges(.obj))),
+            seqn = .seqn,
+            resizeFactor = resizeFactor
+          )
+        )
+      }, obj, this.col.backbone, this.lwd.backbone, names(obj))
+      names(backbone) <- paste0('backbone', names(obj))
+      geometries <- c(geometries, backbone)
+      obj <- unlist(GRangesList(obj))
     }
     
     ## obj is the GRanges with p0 and p1 (x,y,z) coordinates
-
-    geometries <- list() ## list to save all geometries to plot
-    geometries$backbone <- threeJsGeometry(
-      x = c(obj$x0, obj$x1[length(obj)]),
-      y = c(obj$y0, obj$y1[length(obj)]),
-      z = c(obj$z0, obj$z1[length(obj)]),
-      colors = col.backbone,
-      type = "line",
-      tag = "backbone",
-      properties = list(
-        size = lwd.backbone,
-        target = unname(as.character(ranges(obj))),
-        seqn = seqn
-      )
-    )
+    if(cluster3Dpoints){
+      type <- ifelse('clusterType' %in% names(dots),
+                     ifelse(dots$clusterType %in% c('sphere', 'segment'), 
+                            dots$clusterType, 'sphere'), 'sphere')
+      pc_geometries <-
+        createPointClusterGeometries(pc, obj,
+                                     resizeFactor = resizeFactor, type=type)
+    }
 
     rate <- 50
 
@@ -317,6 +382,7 @@ view3dStructure <- function(obj, feature.gr,
         color = col.backbone_background,
         lwd.maxGenomicSigs = lwd.maxGenomicSigs,
         alpha = alpha.backbone_background,
+        resizeFactor = resizeFactor,
         ...,
         SIMPLIFY = FALSE
       )
@@ -330,7 +396,7 @@ view3dStructure <- function(obj, feature.gr,
       geometries <- geometries[lengths(geometries) > 0]
     }
     ## add genomic coordinates
-    if (show_coor) {
+    if (show_coor & singleChr) {
       r_tick <- range(obj)
       end(r_tick) <- ceiling(end(r_tick) / coor_tick_unit) * coor_tick_unit
       start(r_tick) <- floor(start(r_tick) / coor_tick_unit) * coor_tick_unit
@@ -346,7 +412,7 @@ view3dStructure <- function(obj, feature.gr,
         type = "segment",
         colors = col.tension_line,
         tag = "tick_minor",
-        properties = list(size = lwd.tension_line)
+        properties = list(size = lwd.tension_line, resizeFactor = resizeFactor)
       )
 
       if (coor_mark_interval) {
@@ -362,7 +428,8 @@ view3dStructure <- function(obj, feature.gr,
             type = "segment",
             colors = col.tension_line,
             tag = "tick_major",
-            properties = list(size = lwd.tension_line)
+            properties = list(size = lwd.tension_line,
+                              resizeFactor = resizeFactor)
           )
           coor_text <- prettyMark(
             start(feature.tick.mark),
@@ -380,7 +447,8 @@ view3dStructure <- function(obj, feature.gr,
                 label = coor_text[idx],
                 size = .1,
                 depth = .02,
-                pos = 3
+                pos = 3,
+                resizeFactor = resizeFactor
               )
             )
           })
@@ -412,8 +480,9 @@ view3dStructure <- function(obj, feature.gr,
               type = "line",
               tag = feature_tag,
               properties = list(
-                size = lwd.gene,
-                target = unname(genePos$fgf$label[idx])
+                size = ifelse(is.numeric(genePos$fgf$lwd[idx]), genePos$fgf$lwd[idx], lwd.gene),
+                target = unname(genePos$fgf$label[idx]),
+                resizeFactor = resizeFactor
               )
             )
           })
@@ -431,7 +500,8 @@ view3dStructure <- function(obj, feature.gr,
               type = "segment",
               tag = "tss_labels",
               properties = list(
-                size = lwd.gene / 2
+                size = lwd.gene / 2,
+                resizeFactor = resizeFactor
               )
             )
             isGene <- tolower(genePos$fgf$type) %in% "gene" &
@@ -450,7 +520,8 @@ view3dStructure <- function(obj, feature.gr,
                     headLength = as.numeric(arrowLen) * .2,
                     headWidth = as.numeric(arrowLen) * .2,
                     size = lwd.gene / 2,
-                    target = unname(genePos$fgf$label[idx])
+                    target = unname(genePos$fgf$label[idx]),
+                    resizeFactor = resizeFactor
                   )
                 )
               })
@@ -488,7 +559,8 @@ view3dStructure <- function(obj, feature.gr,
                   colors = genePos$fgf$col[notGene],
                   tag = feature_tag,
                   properties = list(
-                    radius = notGeneRadius[1]
+                    radius = notGeneRadius[1],
+                    resizeFactor = resizeFactor
                   )
                 )
               } else {
@@ -503,7 +575,8 @@ view3dStructure <- function(obj, feature.gr,
                     colors = genePos$fgf$col[notGene][idx],
                     tag = feature_tag,
                     properties = list(
-                      radius = .r
+                      radius = .r,
+                      resizeFactor = resizeFactor
                     )
                   )
                 })
@@ -528,7 +601,8 @@ view3dStructure <- function(obj, feature.gr,
                     label = unname(genePos$fgf$label[idx]),
                     size = 0.1,
                     depth = 0.02,
-                    pos = 4
+                    pos = 4,
+                    resizeFactor = resizeFactor
                   )
                 )
               }
